pub type MessageSender<M> = Sender<M, SmartChannelId>;Expand description
Type alias for the receivers returned by the get_sender method of the Hub
Aliased Type
pub struct MessageSender<M> { /* private fields */ }pub type MessageSender<M> = Sender<M, SmartChannelId>;Type alias for the receivers returned by the get_sender method of the Hub
pub struct MessageSender<M> { /* private fields */ }