pub struct SmartChannelId { /* private fields */ }Expand description
SmartChannelId is a unique identifier for channels within a NotifierHub.
It consists of a monotonically increasing counter and the memory address of the NotifierHub
(converted to usize). This guarantees that the ID is unique across different contexts.
The address represents a specific field of a specific NotifierHub, ensuring its global uniqueness.
We store the address as a usize instead of a raw pointer to simplify the type and to keep this type simple without involving generics.